$:.unshift File.join(File.dirname(__FILE__), "..", "lib")

require 'test/unit'
require 'test_generic'

class TC_TestAdminunit < TestGeneric

  require 'crdb/adminunit'

  connect_info = TestGeneric.make_dev_session()

  # Initialize
  ClassName = 'CRDB::Adminunit' 
  classname = TestGeneric.get_classname(ClassName)
  classname.initialize_fake_method_to_value_hash()

  # Clean up any left-over fake records
  classname.delete_all_fakes()

  @@fake_m2v_hash = nil
  def setup
    @@fake_m2v_hash = TestGeneric.get_fake_method_to_value_hash(ClassName) 
  end

  # TEST: Create an object
  def test_00_new
    X_test_new(ClassName)
  end

  # TEST: Create an object and populate it
  def test_01_populate
    X_test_populate(ClassName, 'long_name', @@fake_m2v_hash['long_name'][0])
  end

  # TEST: Create an object and save it (then delete it). 
  def test_02_create
    X_test_create(ClassName, 'long_name', @@fake_m2v_hash['long_name'][0])
  end

  # TEST: Test uniqueness of some of the fields.
  def test_03_uniqueness
    X_test_field_uniqueness(ClassName, 'code')
  end

  # TEST: Try and save with with some validation errors.
  def test_04_validation_tests
    bad_codes = [
                 '',            # Empty
                 'A', 
                 'AB', 
                 'ABC', 
                 'ABCDE', 
                 'ABC^', 
                 'AB D', 
                 '0ABC', 
                ] 
    X_test_field_validity(ClassName, 'code', bad_codes)
  end


  # TEST: Test parsing of a comma-separated string of adminunit codes.
  def test_05_parse_adminunit_string
    adminunits = (CRDB::Adminunit).find(:all) 

    # Get the first three.
    adminunit0 = adminunits[0] 
    adminunit1 = adminunits[1] 
    adminunit2 = adminunits[2] 

    code0 = adminunit0.code
    code1 = adminunit1.code
    code2 = adminunit2.code

    adminunits_string = "#{code0}, #{code1}  ,#{code2}, ,,, "

    admin_units_array = (CRDB::Adminunit).parse_adminunits_string(adminunits_string)
    assert(admin_units_array.length == 3, 'wrong number of adminunits returned')

    assert(admin_units_array[0].code == code0, 'incorrect code')
    assert(admin_units_array[1].code == code1, 'incorrect code')
    assert(admin_units_array[2].code == code2, 'incorrect code')
  end

  # TEST: Test converting an array of CRDB::Adminunit objects into a single
  # string of codes. 
  def test_06_parse_format_multiple_adminunits
    adminunits = (CRDB::Adminunit).find(:all) 

    # Get the first three.
    adminunits3 = adminunits[0, 3]

    adminunit0 = adminunits3[0] 
    adminunit1 = adminunits3[1] 
    adminunit2 = adminunits3[2] 

    code0 = adminunit0.code
    code1 = adminunit1.code
    code2 = adminunit2.code

    formatted_string = (CRDB::Adminunit).format_multiple_adminunits(adminunits3)
    adminunits_string = "#{code0}-#{code1}-#{code2}"

    assert(adminunits_string == formatted_string, "wrong formatted string; is: " +
                      "#{formatted_string} but should have been #{adminunits_string}")
  end

  # TEST: Test getting adminunit objects from the admin code web page.
  def test_07_get_codes_from_adminurl
    adminunits = (CRDB::Adminunit).get_adminunits_from_url()
  
    assert(adminunits.length > 1000, 
            "not enough units returned; should be at least 1000")
  end

end

