$:.unshift File.join(File.dirname(__FILE__), "..", "lib")

require 'test/unit'
require 'test_generic'

class TC_TestBuilding < TestGeneric

  require 'crdb/building'

  connect_info = TestGeneric.make_dev_session()

  # Initialize
  ClassName = 'CRDB::Building' 
  classname = TestGeneric.get_classname(ClassName)
  classname.initialize_fake_method_to_value_hash()

  # Clean up any left-over fake records
  classname.delete_all_fakes()

  @@fake_m2v_hash = nil
  def setup
    @@fake_m2v_hash = TestGeneric.get_fake_method_to_value_hash(ClassName) 
  end


  # TEST: Create an object
  def test_00_new
    X_test_new(ClassName)
  end

  # TEST: Create an object and populate it
  def test_01_populate
    X_test_populate(ClassName, 'full_name', @@fake_m2v_hash['full_name'][0])
  end

  # TEST: Create an object and save it (then delete it). 
  def test_02_create
    X_test_create(ClassName, 'full_name', @@fake_m2v_hash['full_name'][0])
  end

  # TEST: Test uniqueness of some of the fields.
  def test_03_uniqueness
    X_test_field_uniqueness(ClassName, 'QB')
    X_test_field_uniqueness(ClassName, 'full_name')
    X_test_field_uniqueness(ClassName, 'short_name')
    X_test_field_uniqueness(ClassName, 'very_short_name')
  end

  # TEST: Try and save with with some validation errors.
  def test_04_validation_tests
    bad_QBs = [
                 '',            # Empty
                 '-001', 
                 '9-001', 
                 'X9-001', 
                 '999-001', 
                 '99001', 
                 '99-01', 
                 '99-0', 
                 '99-', 
                 '99', 
                 '99-T001', 
                 '99-A00', 
                 '99-001AB', 
                ] 
    X_test_field_validity(ClassName, 'QB', bad_QBs)

   bad_very_short_names = [
                            '012345678901234567890123456789012',
                          ] 
    X_test_field_validity(ClassName, 'very_short_name', bad_very_short_names)
  end

  # TEST: Test the (CRDB::Building).parseQBFR class method.
  def test_05_test_Building_parseQBFR_method
    qbfr_tests = {
             '00-999T-GR-M016A' => ['00-999T', 'GR-M016A'],
             '00-999T-01-4'     => ['00-999T', '01-4'],
             '00-999T-01-BRZWY' => ['00-999T', '01-BRZWY'],
             '00-999T-01'       => ['00-999T', '01'],
             '00-999T'          => ['00-999T', nil],
             '00'               => ['00', nil],
             ''                 => [nil, nil],
             nil                => [nil, nil],
                 } 
    qbfr_tests.each do |qbfr, results|
      qb, fr = (CRDB::Building).parseQBFR(qbfr)
      assert(((qb == results[0]) && (fr == results[1])), 'QBFR parse error')
    end
  end

  # TEST: We overrode the eql? method so test it. 
  def test_06_eql_override
    obj = (CRDB::Building).new()
    fake0 = obj.make_fake({:index => 0})
    fake1 = obj.make_fake({:index => 1})

    assert(!fake0.eql?(fake1), 'eql? failed with two different buildings')
    assert(fake0.eql?(fake0), 'eql? failed when object are the same')

    fake0.QB = fake1.QB
    fake0.full_name = fake1.full_name
    fake0.short_name = fake1.short_name
    fake0.very_short_name = fake1.very_short_name
    assert(fake0.eql?(fake0), 'eql? failed when objects are the same except id')
  end


end