require 'test/unit'
require 'test_generic'

class TC_TestCrdbRemctl < TestGeneric

  require 'crdb/crdb_remctl'

  connect_info = TestGeneric.make_dev_session()

  # TEST: Create an object
  def test_00_new
    crdb_remctl = (CRDB::CRDBRemctl).new
    assert(CRDB::CRDBRemctl, crdb_remctl)
  end

  # TEST: Get the help screen
  def test_10_help
    crdb_remctl = (CRDB::CRDBRemctl).new
    help = crdb_remctl.help()
    assert_match(/DELETE/, help)
    assert_match(/list locations/, help)
  end

 # TEST: show
  def test_20_show
    resource = TestGeneric.create_resource()

    ARGV[0] = 'show'
    ARGV[1] = '-p'
    ARGV[2] = 'dev'
    ARGV[3] = resource.uid

    crdb_remctl = (CRDB::CRDBRemctl).new
    show_text = crdb_remctl.show()
    assert_match(/uid:\s+#{resource.uid}/, show_text)

    TestGeneric.delete_resource(resource)
  end

  # TEST: find
  def test_30_find
    # Make a fake resource
    resource = TestGeneric.create_resource()

    # ###     # ###     # ###     # ###     # ###     # ###     # ###
    find_this = lambda { |fieldname, string, match_string|
      ARGV[0] = 'find'
      ARGV[1] = '-p'
      ARGV[2] = 'dev'
      if (!fieldname.nil?)
        ARGV[3] = fieldname
      end
      if (!string.nil?)
        ARGV[4] = string
      end

      crdb_remctl = (CRDB::CRDBRemctl).new
      find_text = crdb_remctl.find()
      assert_match(/#{match_string}/, find_text)
    }
    # ###     # ###     # ###     # ###     # ###     # ###     # ###

    # 1. uid
    find_this.call('uid', 'cal_lo_fake', 'fake')

    # 2. building
    find_this.call('building', 'Alfred Newman', 'cal_lo_fake')

    # 3. floor-room
    find_this.call('floor-room', '111', 'cal_lo_fake')

    # 4. contact_phone
    find_this.call('contact_phone', '555', 'cal_lo_fake')

    # No result
    find_this.call('contact_phone', 'QQQ', 'no matching resources found')

    # Invalid fieldname
    assert_raise(RuntimeError) {
      find_this.call('QQQ', 'QQQ', 'QQQ')
    }

    # No arguments
    find_this.call(nil, nil, 'Available fields to search')

    TestGeneric.delete_resource(resource)
  end

  # TEST: list
  def test_40_find

    resource = TestGeneric.create_resource()

    # ###     # ###     # ###     # ###     # ###     # ###     # ###
    find_this = lambda { |scope|
      ARGV[0] = 'list'
      ARGV[1] = '-p'
      ARGV[2] = 'dev'
      ARGV[3] = scope

      crdb_remctl = (CRDB::CRDBRemctl).new
      list_text = crdb_remctl.list()
      assert_match(/#{resource.uid}/, list_text)
    }
    # ###     # ###     # ###     # ###     # ###     # ###     # ###

    find_this.call('all')
    find_this.call('locations')

    TestGeneric.delete_resource(resource)
  end


end
