$:.unshift File.join(File.dirname(__FILE__), "..", "lib")

require 'test/unit'
require 'test_generic'

class TC_TestDepartment < TestGeneric

  require 'crdb/department'

  connect_info = TestGeneric.make_dev_session()

  # Initialize
  ClassName = 'CRDB::Department' 
  classname = TestGeneric.get_classname(ClassName)
  classname.initialize_fake_method_to_value_hash()

  # Clean up any left-over fake records
  #puts classname.inspect
  classname.delete_all_fakes()

  @@fake_m2v_hash = nil
  def setup
    @@fake_m2v_hash = TestGeneric.get_fake_method_to_value_hash(ClassName) 
  end

  # TEST: Create an object
  def test_00_new
    X_test_new(ClassName)
  end

  # TEST: Create an object and populate it
  def test_01_populate
    X_test_populate(ClassName, 'long_name', @@fake_m2v_hash['long_name'][0])
  end

  # TEST: Create an object and save it (then delete it). 
  def test_02_create
    X_test_create(ClassName, 'long_name', @@fake_m2v_hash['long_name'][0])
  end

  # TEST: Test uniqueness of some of the fields.
  def test_03_uniqueness
    X_test_field_uniqueness(ClassName, 'code')
    X_test_field_uniqueness(ClassName, 'long_name')
  end

  # TEST: Try and save with with some validation errors.
  def test_04_validation_tests
    bad_codes = [
                 '',            # Empty
                 'LS K', 
                 'ABC*&^', 
                ] 
    X_test_field_validity(ClassName, 'code', bad_codes)
  end

   # TEST: Test parsing of a comma-separated string of department codes.
  def test_05_parse_department_string
    depts = (CRDB::Department).find(:all) 

    # Get the first three.
    dept0 = depts[0] 
    dept1 = depts[1] 
    dept2 = depts[2] 

    code0 = dept0.code
    code1 = dept1.code
    code2 = dept2.code

    depts_string = "#{code0}, #{code1}  ,#{code2}, ,,, "

    departments = (CRDB::Department).parse_departments_string(depts_string)
    assert(departments.length == 3, 'wrong number of deparments returned')

    assert(departments[0].code == code0, 'incorrect code')
    assert(departments[1].code == code1, 'incorrect code')
    assert(departments[2].code == code2, 'incorrect code')
  end

  # TEST: Test converting an array of (CRDB::Department) objects into a single
  # string of codes. 
  def test_06_parse_format_multiple_departments
    depts = (CRDB::Department).find(:all) 

    # Get the first three.
    depts3 = depts[0, 3]

    dept0 = depts3[0] 
    dept1 = depts3[1] 
    dept2 = depts3[2] 

    code0 = dept0.code
    code1 = dept1.code
    code2 = dept2.code

    formatted_string = (CRDB::Department).format_multiple_departments(depts3)
    depts_string = "#{code0}-#{code1}-#{code2}"

    assert(depts_string == formatted_string, "wrong formatted string; is: " +
                      "#{formatted_string} but should have been #{depts_string}")
  end

end
