require 'test/unit'
require 'crdb/session'

class TestGeneric  < Test::Unit::TestCase

  
  def X_test_new(class_name)
    obj = class_name.constantize.new() 
    fake = obj.make_fake()
    assert(fake.class.to_s == class_name, 'Assertion was false.')
  end

  def X_test_populate(class_name, method, value)
    obj = class_name.constantize.new() 
    fake = obj.make_fake()

    assert(fake.class.to_s == class_name, 'Assertion was false.')
    assert(fake.send(method) == value, 'Assertion was false.')
  end

  # TEST: Create an object and save it (then delete it). 
  def X_test_create(class_name, method, value)
    obj = class_name.constantize.new() 
    fake = obj.make_fake()
    fake.save!

    if (value.nil?)
      value = fake.send(method)
    end

    obj_new = class_name.constantize.find(:first, :conditions => {method => value})

    if (obj_new.nil?)
      raise "could not find record using method '#{method}', value '#{value}'"
    end
    assert(obj_new.send(method) == value, 'Assertion was false.')
    fake.destroy()
    obj_new = class_name.constantize.find(:first, :conditions => {method => value})
    assert(obj_new.nil?, 'Assertion was false.')
  end

  # TEST: Create an object and then another object with the same value in
  # a field with a uniqueness constraint.
  def X_test_field_uniqueness(class_name, method)
    obj = class_name.constantize.new() 
    fake1 = obj.make_fake({:index => 0})

    fake1.save!

    # Make the second object but use the second index
    fake2 = obj.make_fake({:index => 1})
 
    # Now change the field given in 'method' to agree with the first fake
    # object. 
    fake2.send(method + '=', fake1.send(method))

    # Attempt to save (should raise an exception)
    begin
      fake2.save!

    rescue
      assert(true, 'record should not have saved!')
    else
      assert(false, 'we should not get here!')
    end

    # Cleanup
    fake1.destroy()
    fake2.destroy()
  end

  # TEST: Take a field and an array of values for the field that are
  # invalid and attempt to save.
  def X_test_field_validity(class_name, method, bad_values)
    obj = class_name.constantize.new() 
    fake = obj.make_fake()

    bad_values.each do |bad_value|
      fake.send(method + '=', bad_value)
      assert_raise(ActiveRecord::RecordInvalid) { fake.save! }
    end
  end

  def X_test_create_resource_from_xml_file(resource_filepath)
    # Step 1. Slurp the xml string
    xml_string = (CRDB::Misc).slurp(resource_filepath)

    # Step 2. Create the resource object
    resource = (CRDB::Resource).from_xml(xml_string)

    # Step 3. Return the resource
    return resource
  end

  def test_nothing
    assert(true, '')
  end


#  # TEST: Create an object and then another object with the same value in
#  # a field with a uniqueness constraint.
#  def X_test_two_field_uniqueness(class_name, method1, method2)
#    obj = class_name.constantize.new() 
#    fake0 = obj.make_fake({:index => 0})
#
#    fake0.save!
#
#    # Make the second object but use the second index
#    fake1 = obj.make_fake({:index => 1})
# 
#    # Now change the fields given in 'method1' and 'method2' to agree with the first fake
#    # object. 
#    fake1.send(method1 + '=', fake0.send(method1))
#    fake1.send(method2 + '=', fake0.send(method2))
#    puts "fake0" + fake0.inspect()
#    puts "fake1" + fake1.inspect()
#
#    # Attempt to save (should raise an exception)
#    begin
#      fake1.save!
#    rescue
#      assert(true, 'record should not have saved!')
#    else
#      assert(false, 'we should not get here!')
#    end
#
#    # Cleanup
#    fake0.destroy()
#    fake1.destroy()
#  end


  def TestGeneric.get_classname(classname_string)
    object = classname_string.constantize.new() 
    classname = object.class
    return classname
  end

  def TestGeneric.get_fake_method_to_value_hash(classname_string)
    classname = TestGeneric.get_classname(classname_string)
    fake_method_to_value = classname.get_fake_method_to_value_hash()
  end

  def TestGeneric.make_dev_session()
    return (CRDB::Session).connect('development') 
  end

  # Make and delete a resource
  def TestGeneric.create_resource()
    r = (CRDB::Resource).new()
    resource = r.make_fake()
    resource.save!
    return resource
  end

  def TestGeneric.delete_resource(resource)
    resource.destroy
    return
  end
end