require 'test/unit'

require 'crdb/misc'
require 'crdb/crdb_remctl'

class TC_TestMisc < Test::Unit::TestCase

  def test_010_nilorempty
    assert((CRDB::Misc).nilorempty?(nil))
    assert((CRDB::Misc).nilorempty?(''))
  end

  def test_020_nillify
    assert_equal((CRDB::Misc).nillify('A'), 'A')
    assert_nil((CRDB::Misc).nillify(nil))
    assert_nil((CRDB::Misc).nillify(''))
  end

  def test_030_strip_or_nillify
    assert_equal((CRDB::Misc).strip_or_nillify('A'), 'A')
    assert_equal((CRDB::Misc).strip_or_nillify('A  '), 'A')
    assert_equal((CRDB::Misc).strip_or_nillify('   A'), 'A')
    assert_equal((CRDB::Misc).strip_or_nillify('   A '), 'A')
    assert_nil((CRDB::Misc).strip_or_nillify(nil))
    assert_nil((CRDB::Misc).strip_or_nillify(''))
  end

  def test_040_arg_process_platform_option
    assert_equal((CRDB::Misc).arg_process_platform_option('development'), 'development')
    assert_equal((CRDB::Misc).arg_process_platform_option('develop'), 'development')
    assert_equal((CRDB::Misc).arg_process_platform_option('dev'), 'development')
    assert_equal((CRDB::Misc).arg_process_platform_option('dEvE'), 'development')
    assert_raise(RuntimeError) { (CRDB::Misc).arg_process_platform_option('de') }
    assert_raise(RuntimeError) { (CRDB::Misc).arg_process_platform_option('') }
    assert_raise(RuntimeError) { (CRDB::Misc).arg_process_platform_option(nil) }
    assert_equal((CRDB::Misc).arg_process_platform_option('production'), 'production')
    assert_equal((CRDB::Misc).arg_process_platform_option('product'), 'production')
    assert_equal((CRDB::Misc).arg_process_platform_option('prod'), 'production')
    assert_equal((CRDB::Misc).arg_process_platform_option('PrOd'), 'production')
    assert_raise(RuntimeError) { (CRDB::Misc).arg_process_platform_option('pro') }
    assert_raise(RuntimeError) { (CRDB::Misc).arg_process_platform_option('pr') }
    assert_raise(RuntimeError) { (CRDB::Misc).arg_process_platform_option('') }
    assert_raise(RuntimeError) { (CRDB::Misc).arg_process_platform_option(nil) }
  end

  def test_050_sunetid
    # Be sure and supply a valid sunetid
    valid_uid = 'adamhl'
    invalid_uid = 'adamhlQQQQQQ'

    # Make a kerberos object.
    kerberos = (CRDB::CRDBRemctl).make_kerberos()

    assert((CRDB::Misc).sunetid?(valid_uid, {:kerberos => kerberos}))
    assert_equal((CRDB::Misc).sunetid?(invalid_uid, {:kerberos => kerberos}), false)
    assert_raise(RuntimeError) { (CRDB::Misc).sunetid?(nil) }
  end

  def test_060_sunetid_to_uid
    sunetid1 = 'adamhl'
    # Make a kerberos object.
    kerberos = (CRDB::CRDBRemctl).make_kerberos()
    assert_equal((CRDB::Misc).sunetid_to_uid(sunetid1, {:kerberos => kerberos}), 'adamhl')
  end

  def test_070_sunetid_to_uid
    filename = "./slurp_test.txt"
    s = (CRDB::Misc).slurp(filename)
    assert_match(/Hello/, s)
    assert_match(/Goodbye/, s)
  end

  def test_080_sing_or_plural
    assert_equal((CRDB::Misc).sing_or_plural(0), 's')
    assert_equal((CRDB::Misc).sing_or_plural(1), '')
    assert_equal((CRDB::Misc).sing_or_plural(2), 's')
    assert_equal((CRDB::Misc).sing_or_plural(200.5), 's')
  end  

  def test_090_get_crdb_version
    version = (CRDB::Misc).get_crdb_version()
    assert_match(/\d+\.\d+\.\d+/, version)
  end  

end


