$:.unshift File.join(File.dirname(__FILE__), "..", "lib")

require 'test/unit'
require 'test_generic'

class TC_TestPolicy < TestGeneric

  require 'crdb/policy'

  connect_info = TestGeneric.make_dev_session()

  # Initialize
  ClassName = 'CRDB::Policy' 
  classname = TestGeneric.get_classname(ClassName)
  classname.initialize_fake_method_to_value_hash()

  # Clean up any left-over fake records
  classname.delete_all_fakes()

  @@fake_m2v_hash = nil
  def setup
    @@fake_m2v_hash = TestGeneric.get_fake_method_to_value_hash(ClassName) 
  end

  # TEST: Create an object
  def test_00_new
    X_test_new(ClassName)
  end

  # TEST: Create an object and populate it
  def test_01_populate
    X_test_populate(ClassName, 'name', @@fake_m2v_hash['name'][0])
  end

  # TEST: Create an object and save it (then delete it). 
  def test_02_create
    X_test_create(ClassName, 'name', @@fake_m2v_hash['name'][0])
  end

  # TEST: Test uniqueness of some of the fields.
  def test_03_uniqueness
    X_test_field_uniqueness(ClassName, 'name')
    X_test_field_uniqueness(ClassName, 'description')
  end

  # TEST: Try and save with a code in the wrong format
  def test_04_code_wrong_format
    bad_names = [
                 '',            # Empty
                 'aaa_AAA_bbb', # Not all lower-case
                 'abc_#$@',     # Contains non-word characters
                ] 
    X_test_field_validity(ClassName, 'name', bad_names)
  end

  # TEST: We overrode the eql? method so test it. 
  def test_05_eql_override
    obj = (CRDB::Policy).new()
    fake0 = obj.make_fake({:index => 0})
    fake1 = obj.make_fake({:index => 1})

    assert(!fake0.eql?(fake1), 'eql? failed with two different policies')
    assert(fake0.eql?(fake0), 'eql? failed when object are the same')

    fake0.name = fake1.name
    fake0.description = fake1.description
    assert(fake0.eql?(fake0), 'eql? failed when objects are the same except id')
  end


end