$:.unshift File.join(File.dirname(__FILE__), "..", "lib")

require 'test/unit'
require 'test_generic'

class TC_TestResource < TestGeneric

  require 'crdb/resource'

  connect_info = TestGeneric.make_dev_session()

  # Initialize
  ClassName = 'CRDB::Resource'
  classname = TestGeneric.get_classname(ClassName)
  classname.initialize_fake_method_to_value_hash()

  # Clean up any left-over fake records
  classname.delete_all_fakes()

  @@fake_m2v_hash = nil
  def setup
    @@fake_m2v_hash = TestGeneric.get_fake_method_to_value_hash(ClassName)
  end

  # TEST: Create an object
  def test_00_new
    X_test_new(ClassName)
  end

  # TEST: Create an object and populate it
  def test_01_populate
    X_test_populate(ClassName, 'uid', @@fake_m2v_hash['uid'][0])
  end

  # TEST: Create an object and save it (then delete it).
  def test_02_create
    X_test_create(ClassName, 'uid', @@fake_m2v_hash['uid'][0])
  end

  # TEST: Try and save some fields in the wrong format
  def test_04_code_wrong_format
    bad_uids = [
                 '',      # Empty
                 'aaAA',
                 'dfsdfjkl&39%',
                 'cal_lo_abcdefghijklmnopqrstuvwxyz' + ('0' * 30), # Too long
                ]
    X_test_field_validity(ClassName, 'uid', bad_uids)

    bad_phones = [
                 '(650) 723-3472',
                 '723-3472',
                 '650-723-3472 x882',
                ]
    X_test_field_validity(ClassName, 'contact_phone', bad_phones)

    bad_emails = [
                 'adamhl',
                 'adam hl@stanford.edu',
                 'adamhl@stanford.',
                 'adamhl@s**tanford.edu',
                ]
    X_test_field_validity(ClassName, 'contact_email', bad_emails)

    bad_FRs = [
                 'AA',
                 '01-*(*(',
                 '01-01-01',
                ]
    X_test_field_validity(ClassName, 'FR', bad_FRs)

    bad_capacities = [
                 'ABC',
                ]
    X_test_field_validity(ClassName, 'capacity', bad_capacities)

  end

  # TEST: update a record to see if the update observer is fired.
  def test_10_update
    r = (CRDB::Resource).new()
    resource = r.make_fake()
    resource.save!
    resource.room_override = "Plaza 123"
    resource.contact_fullname = "Snerd Merfel"
    resource.save!
    resource.destroy
  end

  # TEST: Create an object from an XML file.
  def test_11_create_from_xml
    X_test_create_resource_from_xml_file('./resource.xml')
  end

  # TEST: Try to create a resource from an XML file that is a non-location
  # and is missing a department.
  def test_12_create_from_xml_missing_department
    assert_raise RuntimeError do
      X_test_create_resource_from_xml_file('./resource_bad_missing_department.xml')
    end
  end

  # TEST: Try to create a resource from an XML file that is a non-location
  # and is missing a resource name.
  def test_13_create_from_xml_missing_resource_name
    assert_raise RuntimeError do
      X_test_create_resource_from_xml_file('./resource_bad_missing_resource_name.xml')
    end
  end

  # TEST: Try to create a resource from an XML file that has a type and
  # category inconsistency.
  def test_14_create_from_xml_type_category_inconsistency
    assert_raise RuntimeError do
      X_test_create_resource_from_xml_file('./resource_bad_type_category.xml')
    end
  end

#  # TEST: Try to change the old_sundial_name.
#  def test_15_ensure_old_sundial_readonly
#    r = (CRDB::Resource).new()
#    resource = r.make_fake()
#    resource.save!
#    assert_nil(resource.old_sundial_name)
#    resource.old_sundial_name = "Polya Hall 455"
#    resource.save
#    resource.reload
#    assert_nil(resource.old_sundial_name)
#    resource.destroy
#  end

  # TEST: CLEANUP
  def test_99_cleanup
    (CRDB::Resource).delete_all_fakes()
  end

end
