$:.unshift File.join(File.dirname(__FILE__), "..", "lib")

require 'test/unit'
require 'test_generic'

class TC_TestResourcephone < TestGeneric

  require 'crdb/resourcephone'

  connect_info = TestGeneric.make_dev_session()

  # Initialize
  ClassName = 'CRDB::Resourcephone' 
  classname = TestGeneric.get_classname(ClassName)
  classname.initialize_fake_method_to_value_hash()

  # Clean up any left-over fake records
  classname.delete_all_fakes()

  @@fake_m2v_hash = nil
  def setup
    @@fake_m2v_hash = TestGeneric.get_fake_method_to_value_hash(ClassName) 
  end

  # TEST: Create an object
  def test_00_new
    X_test_new(ClassName)
  end

  # TEST: Create an object and populate it
  def test_01_populate
    X_test_populate(ClassName, 'resource_id', @@fake_m2v_hash['resource'][0].id)
  end

  # TEST: Create an object and save it (then delete it). 
  def test_02_create
    X_test_create(ClassName, 'resource_id', @@fake_m2v_hash['resource'][0].id)
  end


  # TEST: Try and add a resourcephone to a nonlocation resource. An
  # exception should be raised. 
  def test_15_add_to_nonlocation
    r = (CRDB::Resource).new()
    resource = r.make_fake({:index => 1})
    resource.save!
    resource.reload
    phonenumber = '650-555-4321'
    resource.resourcephones.create(
      :phone => phonenumber
    ) 

    assert_raise(ActiveRecord::RecordInvalid) { resource.save! }

    # Just in case:
    classname = TestGeneric.get_classname(ClassName)
    classname.delete_all_fakes()
  end

  # TEST: Attempt to add an invalid phone number.
  # exception should be raised. 
  def test_20_invalid_phonenumber
    r = (CRDB::Resource).new()
    resource = r.make_fake({:index => 0}) # Index 0 is the location resource.
    resource.save!
    resource.reload
    phonenumber = 'x3-3472'
    resource.resourcephones.create(
      :phone => phonenumber
    ) 

    assert_raise(ActiveRecord::RecordInvalid) { resource.save! }

    # Just in case:
    classname = TestGeneric.get_classname(ClassName)
    classname.delete_all_fakes()
  end


  # TEST: CLEANUP
  def test_99_cleanup
    classname = TestGeneric.get_classname(ClassName)
    classname.delete_all_fakes()
  end

end