$:.unshift File.join(File.dirname(__FILE__), "..", "lib")

require 'test/unit'
require 'test_generic'

class TC_TestType < TestGeneric

  require 'crdb/type'

  connect_info = TestGeneric.make_dev_session()

  # Initialize
  ClassName = 'CRDB::Type' 
  classname = TestGeneric.get_classname(ClassName)
  classname.initialize_fake_method_to_value_hash()

  # Clean up any left-over fake records
  classname.delete_all_fakes()

  @@fake_m2v_hash = nil
  def setup
    @@fake_m2v_hash = TestGeneric.get_fake_method_to_value_hash(ClassName) 
  end

  # TEST: Create an object
  def test_00_new
    X_test_new(ClassName)
  end

  # TEST: Create an object and populate it
  def test_01_populate
    X_test_populate(ClassName, 'code', @@fake_m2v_hash['code'][0])
  end

  # TEST: Create an object and save it (then delete it). 
  def test_02_create
    X_test_create(ClassName, 'code', @@fake_m2v_hash['code'][0])
  end

  # TEST: Test uniqueness of some of the fields.
  def test_03_uniqueness
    X_test_field_uniqueness(ClassName, 'code')
    X_test_field_uniqueness(ClassName, 'description')
  end

  # TEST: Try and save with a code in the wrong format
  def test_04_code_wrong_format
    bad_codes = [
                 '',      # Empty
                 'aaa',   # Not upper-case.
                 'AB123', # Contains non-letters
                ] 
    X_test_field_validity(ClassName, 'code', bad_codes)
  end

  # TEST: test the get_known_type_codes method
  def test_05_test_get_known_type_codes_method
    known_types = CRDB::Type.get_known_type_codes()

    assert(known_types.length > 10, 'too few known type codes')
    assert(known_types.include?('CR'), 'missing CR type code')
  end

end