module CRDB
 module Cache

  # In case this is running on a server where the active record or active
  # support libraries are in /usr/lib/ruby/1.8/: 
  $:.push('/usr/lib/ruby/1.8/')

  require 'rubygems'
  require 'active_support'

  attr :stanford_caching_cache      # The ActiveSupport::Cache::FileStore object
  attr :stanford_caching_cache_dir  # The directory where the cache lives
  attr :stanford_caching_namespace  # A name space to disambiguate 
  attr :stanford_caching_debug  
  attr :stanford_caching_expire_seconds # The number of seconds the cache
                                        # lasts in seconds. (NOT YET IMPLEMENTED)

  def initialize()
    @stanford_caching_cache = nil     # Default cache is nil; if we turn on
                                      # caching, this instance variable will
                                      # become non-nil. 

    @stanford_caching_namespace = nil # User must set namespace to non-nil
                                      # before things will work properly. 

    @stanford_caching_debug = false   # Have debug turned off by default.
  end 


  # Is caching on?
  def caching?()
    if (@stanford_caching_cache.nil?)
      return false
    else
      return true
    end
  end

  # Turn caching on. Returns the cache object.
  def caching_on()
    progress("turning caching ON") 

    # Namespace must be non-nil
    if (@stanford_caching_namespace.nil?)
      raise "cannot turn caching on without setting the namespace"
    end

    # cache dir must be non-nil
    if (@stanford_caching_cache_dir.nil?)
      raise "cannot turn caching on without setting the cache_dir"
    end
    
    # If caching is already on, do nothing
    if (self.caching?)
      return @stanford_caching_cache
    else
      # The directory should be stanford_caching_cache_dir followed by 
      # the namespace. 
      dir = @stanford_caching_cache_dir + @stanford_caching_namespace
      @stanford_caching_cache = ActiveSupport::Cache::FileStore.new(dir)  
      return @stanford_caching_cache
    end
  end

  # Turn caching off. Returns nil
  def caching_off()
    progress("turning caching OFF") 
    @stanford_caching_cache = nil
    return nil
  end

  # Set the caching directory.
  def cache_dir=(c_dir)
    @stanford_caching_cache_dir = c_dir

    # If @stanford_caching_cache does not end in the directory separator,
    # add it now. 
    if (@stanford_caching_cache_dir !~ /#{File::SEPARATOR}$/)
      @stanford_caching_cache_dir = @stanford_caching_cache_dir + File::SEPARATOR
    end

    # If caching is turned on, re-create the cache object in case the
    # cache dir changed. 
    if (self.caching?)
      caching_off() # We turn off so that cache_on will be sure to create
      caching_on()  # a new object. 
    end 
  end

  # Set the caching namespace. If namespace is non-nil, it should consist
  # of only "word" characters (letters, numbers, and underscore).
  def cache_namespace=(namespace)
    if (namespace.nil?)
      @stanford_caching_namespace = nil
    elsif (namespace !~ /^\w+$/)
      raise "namespace '#{namespace}' is invalid; must be only letters, " +
            "numbers, and the underscore" 
    else
      @stanford_caching_namespace = namespace.clone() 
      progress("namespace is #{@stanford_caching_namespace}")
    end 
  end

  # 
  def clear
    if (self.caching?)
      @stanford_caching_cache.delete_matched(/#{@stanford_caching_namespace}/)
    end
  end

  def progress(message)
    if (@stanford_caching_debug)
      $stderr.print(message)
    end
  end

 end # matches  "module Cache"
end # matches "module CRDB"


