module CRDB
 class Department < ActiveRecord::Base

  require 'crdb/resource'
  require 'crdb/ownership'

  # The "join table" ownerships links resources to departments
  has_many :ownerships, :class_name => "CRDB::Ownership"
  has_many :resources, :class_name => "CRDB::Resource", :through => :ownerships

  # Specify some validations
  validates_presence_of :code
  validates_presence_of :long_name
  validates_uniqueness_of :code
  validates_uniqueness_of :long_name
  validates_format_of :code, :with => /\A[a-zA-Z0-9]+\z/


  def to_s
    @id = self.id
    @code = self.code
    @long_name = self.long_name
    return "(id: #{@id}, code: #{@code}, long_name: #{@long_name})"
  end

  # Given an array of CRDB::Department objects, format into a string.
  def (CRDB::Department).format_multiple_departments(array_of_departments)
    codes = []
    array_of_departments.each do |department|
      codes.push(department.code)
    end

    if (codes.empty?)
      return "NO_ORGS"
    else
      return codes.join("-")
    end

  end

  # Given a comma-separated list of departments (e.g., "IT, GSB, AS")
  # parse and return an array of CRDB::Department objects, one for each.
  # If not recognizable departments, returns an empty array.
  # department.
  def (CRDB::Department).parse_departments_string(depts_string)
    if ((CRDB::Misc).nilorempty?(depts_string))
      # If the string is empty or nil, return an empty array.
      return []
    else
      dept_codes = depts_string.split(",")

      # Store the resulting CRDB::Department objects in the array departments.
      departments = []

      dept_codes.each do |dept_code|
        # Strip the code
        dept_code.strip!

        # Skip any empty departments
        if (dept_code.nil? || dept_code.empty?)
          next
        end

        # See if there is a CRDB::Department with this code.
        department = (CRDB::Department).find(:first, :conditions => {:code => dept_code})

        if (!department.nil?)
          # We found one, so add it the results array.
          departments.push(department)
        end
      end
    end

    return departments
  end

  def (CRDB::Department).get_codes()
    depts = (CRDB::Department).find(:all)
    dept_codes = depts.map do |dept|
      dept.code
    end
    return dept_codes
  end

  # ## ###  # ## ###  # ## ###  # ## ###  # ## ###  # ## ###
  # "Mix-in" the Fake module
  require 'crdb/fake.rb'
  include CRDB::Fake

  @@fake_method_to_value_hash = nil
  def (CRDB::Department).initialize_fake_method_to_value_hash()
    fake_codes = ['LSO', 'CITES']
    fake_long_names = ['Library System Office', 'Central IT Support']

    @@fake_method_to_value_hash =
    {
      'code'      => fake_codes,
      'long_name' => fake_long_names,
    }
  end

  def (CRDB::Department).get_fake_method_to_value_hash()
    return @@fake_method_to_value_hash
  end

  def (CRDB::Department).delete_all_fakes()
    (CRDB::Department).initialize_fake_method_to_value_hash()
    fake_long_names = @@fake_method_to_value_hash['long_name']
    (CRDB::Department).delete_all(["long_name = ?", fake_long_names[0]])
    (CRDB::Department).delete_all(["long_name = ?", fake_long_names[1]])
  end
  # ## ###  # ## ###  # ## ###  # ## ###  # ## ###  # ## ###


 end # matches "class Department"
end # matches "module CRDB"
