module CRDB
 class Event < ActiveRecord::Base

  require 'crdb/resource'
  require 'crdb/eventcode'

  attr_accessor :is_fake

  # Here are the foreign keys:
  belongs_to :resource, :class_name => "CRDB::Resource"
  belongs_to :eventcode, :class_name => "CRDB::Eventcode"

  ## VALIDATIONS
  # resource is a foreign key, so it MUST be present and valid
  validates_presence_of :resource, :message => " must be specified"
  validates_associated :resource

  # eventcode is a foreign key, so it MUST be present and valid
  validates_presence_of :eventcode, :message => " must be specified"
  validates_associated :eventcode

  validates_format_of :sunetid, :with => /\A[a-z][a-z0-9]+\z/, :allow_nil => true

  def to_s
    id        = self.id
    eventcode = self.eventcode
    return "(id: #{id}, eventcode: #{eventcode.to_s})"
  end


  # ## ###  # ## ###  # ## ###  # ## ###  # ## ###  # ## ###
  # "Mix-in" the CRDB::Fake module
  require 'crdb/fake.rb'
  include CRDB::Fake

  @@fake_method_to_value_hash = nil
  def (CRDB::Event).initialize_fake_method_to_value_hash()
    (CRDB::Eventcode).delete_all_fakes()
    eventcode = (CRDB::Eventcode).new()
    fake_eventcode0 = eventcode.make_fake({:index => 0})
    fake_eventcode1 = eventcode.make_fake({:index => 1})
    fake_eventcodes = [fake_eventcode0, fake_eventcode1]

    (CRDB::Resource).delete_all_fakes()
    resource = (CRDB::Resource).new()
    fake_resource0 = resource.make_fake({:index => 0})
    fake_resource1 = resource.make_fake({:index => 1})
    fake_resources = [fake_resource0, fake_resource1]

    fake_sunetids = ['adamhl2', nil]

    fake_arg1s = ['Hello, there', 'Goodbye to you']
    fake_arg2s = ['Arrgh!', nil]

    @@fake_method_to_value_hash =
    {
      'eventcode'        => fake_eventcodes,
      'resource'         => fake_resources,
      'sunetid'          => fake_sunetids,
      'arg1'             => fake_arg1s,
      'arg2'             => fake_arg2s,
    }
  end

  def (CRDB::Event).get_fake_method_to_value_hash()
    return @@fake_method_to_value_hash
  end

  def (CRDB::Event).delete_all_fakes()
    (CRDB::Event).initialize_fake_method_to_value_hash()
    fake_arg1s = @@fake_method_to_value_hash['arg1']
    (CRDB::Event).delete_all(["arg1 = ?", fake_arg1s[0]])
    (CRDB::Event).delete_all(["arg1 = ?", fake_arg1s[1]])
    (CRDB::Resource).delete_all_fakes()
    (CRDB::Eventcode).delete_all_fakes()
  end
  # ## ###  # ## ###  # ## ###  # ## ###  # ## ###  # ## ###

 end # matches "class Event"
end # matches "module CRDB"
