module CRDB
  class Eventcode < ActiveRecord::Base

  # Every event has an eventcode
  has_one :event, :class_name => "CRDB::Event"

  # Specify some validations
  validates_uniqueness_of :code

  # The code field must start with 'EV_'.
  validate :code_must_start_EV

  def code_must_start_EV
    if (code !~ /^EV_[A-Z0-9_]+$/)
      errors.add_to_base("Event code must start 'EV_' (#{code})") 
    end
  end

  def to_s
    id   = self.id
    code = self.code
    description = self.description
    return "(id: #{id}, code: #{code}, description: #{description})" 
  end

  # ## ###  # ## ###  # ## ###  # ## ###  # ## ###  # ## ###
  # "Mix-in" the Fake module
  require 'crdb/fake.rb'
  include CRDB::Fake

  @@fake_method_to_value_hash = nil
  def (CRDB::Eventcode).initialize_fake_method_to_value_hash()
    fake_codes = ['EV_FAKE_CODE_001', 'EV_FAKE_CODE_002']
    fake_descriptions = ['A test code: %arg1%', 'Another test code %arg1%, %arg2%']

    @@fake_method_to_value_hash = 
    {
      'code'        => fake_codes,
      'description' => fake_descriptions,
    }

    return @@fake_method_to_value_hash
  end

  def (CRDB::Eventcode).get_fake_method_to_value_hash()
    return @@fake_method_to_value_hash
  end

  def (CRDB::Eventcode).delete_all_fakes()
    (CRDB::Eventcode).initialize_fake_method_to_value_hash()
    fake_codes = @@fake_method_to_value_hash['code']
    (CRDB::Eventcode).delete_all(["code = ?", fake_codes[0]])
    (CRDB::Eventcode).delete_all(["code = ?", fake_codes[1]])
  end   
  # ## ###  # ## ###  # ## ###  # ## ###  # ## ###  # ## ###

 end # matches "module CRDB"
end # matches "class Eventcode"
