require 'tempfile'

module CRDB
 class Kerberos < Object

  attr_accessor :principal_primary,
                :principal_instance,
                :principal_realm,
                :keytab_file,
                :ticket_file

  def initialize()
    @principal_realm = 'stanford.edu'
  end

  def make_kerberos_ticket
    # Does the keytab file exist? If not, die.
    if (!File.exists?(@keytab_file))
      raise "keytab file '#{@keytab_file}' not found"
    end

    # Make a temporary file    
    @ticket_file = Tempfile.new('krbtck') 

    # Make the kerberos ticket 
    cmd = "k5start -u #{@principal_primary} -i #{@principal_instance} " + 
          "-r #{@principal_realm} -f #{@keytab_file} -k #{@ticket_file.path}"
    #puts cmd 

    require "open3"
    stdin, stdout, stderr = Open3.popen3(cmd)

    # Dump output
    stdout_string = stdout.read
    stderr_string = stderr.read
 
    if (stderr_string.length > 0)
      raise "there was error creating the kerberos ticket: " +
            "[#{stderr_string}]" 
    end
   
    # Set the environment variable KRB5CCNAME to point to this ticket file.
    ENV['KRB5CCNAME'] = "FILE:#{@ticket_file.path}"

    return ticket_file
  end

  def run(cmd)
    # Does the keytab file exist? If not, die.
    if (!File.exists?(@keytab_file))
      raise "keytab file '#{@keytab_file}' not found"
    end

    # k5start -t -U -f /etc/keytab.pobox -- sh -c "$PERL /afs/ir/dev/calendar/migration/scripts/IMPORT_WQ.pl 2> $LOGFILE"

    # Escape any " (double-quote) characters.
    cmd.gsub!(/\"/, '\"')
 
    cmd1 = %Q[k5start -t -U -f #{@keytab_file} -- ] +
           %Q[sh -c "#{cmd}"]
    # puts cmd1 

    stdout, stderr = (CRDB::Misc).run_command(cmd1)

    if ((!stderr.nil?) && (!stderr.empty?))
      stderr = stderr + "[cmd: #{cmd1}]" 
    end

    return [stdout, stderr]
  end

 end # matches "class Kerberos"
end # matches "module CRDB"
