module CRDB
 class Ownership < ActiveRecord::Base

  require 'crdb/department'
  require 'crdb/resource'

  # Foreign keys
  belongs_to :resource, :class_name => "CRDB::Resource"
  belongs_to :department, :class_name => "CRDB::Department"

  ## VALIDATIONS
  validates :resource_id, :uniqueness => { :scope => :department_id }
#  validates_presence_of :department, :message => " must be specified"
#  validates_presence_of :resource, :message => " must be specified"
#  validates_uniqueness_of :department_id, :scope => :resource_id
#  validates_uniqueness_of :resource_id, :scope => :department_id

  def to_s
    resource = self.resource
    department = self.department

    if (!resource.nil?)
      resource_formatted = resource.to_s
    else
      resource_formatted = 'NO RESOURCE'
    end

    if (!department.nil?)
      department_formatted = department.to_s
    else
      department_formatted = 'NO OWNING ORGANIZATIONS'
    end

    return resource_formatted + "\n" + department_formatted
  end

  # ## ###  # ## ###  # ## ###  # ## ###  # ## ###  # ## ###
  # "Mix-in" the Fake module
  require 'crdb/fake.rb'
  include CRDB::Fake

  @@fake_method_to_value_hash = nil
  def (CRDB::Ownership).initialize_fake_method_to_value_hash()
    require 'crdb/department'
    (CRDB::Department).delete_all_fakes()
    department = (CRDB::Department).new()
    fake_department0 = department.make_fake({:index => 0})
    fake_department1 = department.make_fake({:index => 1})
    fake_departments = [fake_department0, fake_department1]

    require 'crdb/resource'
    (CRDB::Resource).delete_all_fakes()
    resource = (CRDB::Resource).new()
    fake_resource0 = resource.make_fake({:index => 0})
    fake_resource1 = resource.make_fake({:index => 1})
    fake_resources = [fake_resource0, fake_resource1]

    @@fake_method_to_value_hash =
    {
      'department'     => fake_departments,
      'resource'       => fake_resources,
    }
  end

  def (CRDB::Ownership).get_fake_method_to_value_hash()
    return @@fake_method_to_value_hash
  end

  def (CRDB::Ownership).delete_all_fakes()
    (CRDB::Ownership).initialize_fake_method_to_value_hash()
    fake_departments = @@fake_method_to_value_hash['department']
    (CRDB::Ownership).delete_all(["department_id = ?", fake_departments[0].id])
    (CRDB::Ownership).delete_all(["department_id = ?", fake_departments[1].id])
    (CRDB::Resource).delete_all_fakes()
    (CRDB::Department).delete_all_fakes()
  end
  # ## ###  # ## ###  # ## ###  # ## ###  # ## ###  # ## ###

 end # matches "class Ownership"
end # matches "module CRDB"
