# In case this is running on a server where the active record or active
# support libraries are in /usr/lib/ruby/1.8/:
# $:.push('/usr/lib/ruby/1.8/')

require 'rubygems'
require 'active_support'

require 'crdb/misc'

module CRDB
 class Remctl < Object

  @@CACHE_DIR_DEFAULT = "/tmp/rubycache/remctl"
  @@CACHE_EXPIRES_IN  =  20.seconds

  attr_accessor :server, :type, :service, :options,
                :caching, :cache, :cache_dir, :debug,
                :kerberos



  def initialize
    @caching = false
    @debug = false
    if (@cache_dir.nil?)
      @cache_dir = @@CACHE_DIR_DEFAULT
    end
  end

  def run
    cmd = "remctl #{@server} #{@type} #{@service} #{@options}"

    # If caching is turned on, see if this object is in the cache
    cache_key = "remctl_#{@server}_#{@type}_#{@service}_#{@options}"

    if (@caching)
      progress("CACHING IS ON")
      result = @cache.read(cache_key)
      if (!result.nil?)
        # HIT
        progress("CACHE HIT")
        return result
      else
        progress("CACHE MISS")
      end
    else
      progress("CACHING IS OFF")
    end

    # If we get here, we need to run the command.
    $VERBOSE = false
    # puts cmd
    if (@kerberos)
      stdout, stderr = @kerberos.run(cmd)
    else
      stdout, stderr = (CRDB::Misc).run_command(cmd)
    end

    # If there is an error, raise it now.
    if (!stderr.nil? && !stderr.empty?)
      # We will ignore the "Insecure world writable dir" error
      if (stderr =~ /Insecure world writable dir/)
        # Ignore
      else
        raise "error running command #{cmd}: #{stderr}"
      end
    end

    # If we are caching, cache now.
    # Currently :expires_in option is not supported for
    # ActiveSupport::Cache::FileStore. :^(
    if (@caching)
      @cache.write(cache_key, stdout, :expires_in => @@CACHE_EXPIRES_IN)
    end

    return stdout
  end

  def to_s
    return <<-EOS
server:    #{@server}
type:      #{@type}
service:   #{@service}
options:   #{@options}
kerberos:  #{@kerberos.inspect}
EOS
  end

  def caching?()
    return @caching
  end

  def caching_on()
    progress("turning Remctl caching ON")
    @caching = true
    if (@cache.nil?)
      @cache = ActiveSupport::Cache::FileStore.new(@cache_dir)
    end

    return @caching
  end

  def caching_off()
    progress("turning Remctl caching OFF")
    @caching = false
    @cache = nil

    return @caching
  end

  def cache_dir=(c_dir)
    @cache_dir = c_dir

    # If caching is turned on, re-create the cache
    # object in case the cache dir changed.
    if (@caching)
      @cache = ActiveSupport::Cache::FileStore.new(@cache_dir)
    end
  end

  def clear_cache
    if (@caching)
      @cache.delete_matched(/remctl/)
    end
  end

  def progress(message)
    if (@debug)
      $stderr.print(message)
    end
  end

  def debug_on()
    @debug = true
  end

  def debug_off()
    @debug = false
  end

 end # matches "class Remctl"
end # matches "module CRDB"

