module CRDB
 class Resourceadmin < ActiveRecord::Base

  require 'crdb/resource'

  # There is a foreign key resource_id
  belongs_to :resource, :class_name => "CRDB::Resource"

  ## VALIDATIONS
  # NO! validates_presence_of :resource, :message => " must be specified"

  # At least one of sunetid or workinfo must be nonnull.
  validate :must_have_sunetid_or_workgroup
  def must_have_sunetid_or_workgroup
    if (sunetid.nil? && workgroup.nil?)
      errors.add_to_base("At least one of sunetid or workgroup " +
                         "must be nonnull")
    end
  end

  # Cannot have more than (resource_id, sunetid) pair.
  validate :sunetid_resourceid_unique
  def sunetid_resourceid_unique
    if (!sunetid.nil?)
      # Is there already an entry with this (sunetid, resource_id)
      resourceadmins = (CRDB::Resourceadmin).where("resource_id = ? AND sunetid = ?", resource_id, sunetid)
      if (!resourceadmins.empty?)
        errors.add_to_base("there is already a resourceadmin object " +
                           "with resource_id #{resource_id} " +
                           "and sunetid #{sunetid}")
      end
    end
  end


  def to_s
    resource  = self.resource
    sunetid   = self.sunetid
    workgroup = self.workgroup

    resource_formatted = resource.uid
    return "(id: #{@id}, resource: #{resource_formatted}, sunetid: #{sunetid}, workgroup: #{workgroup})"
  end


  def principal
    if (!self.sunetid.nil?)
      return self.sunetid
    elsif (!self.workgroup.nil?)
      return self.workgroup
    else
      raise "no principal?!?"
    end
  end

  # Given an array of CRDB::Department objects, format into a string.
  def (CRDB::Resourceadmin).format_multiple_resourceadmins(array_of_resourceadmins)
    principals = []
    array_of_resourceadmins.each do |resourceadmin|
      principals.push(resourceadmin.principal)
    end

    if (principals.empty?)
      return "NO_RESOURCE_ADMINS"
    else
      return principals.join(", ")
    end

  end

  # ## ###  # ## ###  # ## ###  # ## ###  # ## ###  # ## ###
  # "Mix-in" the Fake module
  require 'crdb/fake.rb'
  include CRDB::Fake

  @@fake_method_to_value_hash = nil
  def (CRDB::Resourceadmin).initialize_fake_method_to_value_hash()
    require 'crdb/resource'
    (CRDB::Resource).delete_all_fakes()
    resource = (CRDB::Resource).new()
    fake_resource0 = resource.make_fake({:index => 0})
    fake_resource1 = resource.make_fake({:index => 1})
    fake_resources = [fake_resource0, fake_resource1]

    fake_sunetids   = ['adamhl', nil]
    fake_workgroups = [nil, '~adamhl:test_workgroup']

    @@fake_method_to_value_hash =
    {
      'resource'  => fake_resources,
      'sunetid'   => fake_sunetids,
      'workgroup' => fake_workgroups,
    }
  end

  def (CRDB::Resourceadmin).get_fake_method_to_value_hash()
    return @@fake_method_to_value_hash
  end

  def (CRDB::Resourceadmin).delete_all_fakes()
    (CRDB::Resourceadmin).initialize_fake_method_to_value_hash()
    fake_resources = @@fake_method_to_value_hash['resource']
    (CRDB::Resourceadmin).delete_all(["resource_id = ?", fake_resources[0].id])
    (CRDB::Resourceadmin).delete_all(["resource_id = ?", fake_resources[1].id])
    (CRDB::Resource).delete_all_fakes()
  end
  # ## ###  # ## ###  # ## ###  # ## ###  # ## ###  # ## ###

 end # matches "class Resourceadmin"
end # matches "module CRDB"
