module CRDB
 class Resourcephone < ActiveRecord::Base

  require 'crdb/resource'

  # There is a foreign key resource_id
  belongs_to :resource, :class_name => "CRDB::Resource"

  ## VALIDATIONS
  validates_presence_of :phone, :message => " must be specified" 
  #validates_presence_of :resource_id, :message => " must be specified" 
  validates_uniqueness_of :phone, :scope => :resource_id, 
                                  :message => "phone number is not unique"

  validate :phone_is_valid
  def phone_is_valid
    if (!(CRDB::Misc).nilorempty?(phone))
      if (phone !~ /^\d\d\d\-\d\d\d\-\d\d\d\d$/)
        msg = "resource phone #{phone} looks to be invalid"
        errors.add_to_base(msg) 
      end
    end
  end 


  # Given an array of CRDB::Resourcephone objects, format into a string.
  def (CRDB::Resourcephone).format_multiple_resourcephones(array_of_resourcephones)
    phones = []
    array_of_resourcephones.each do |resourcephone|
      phones.push(resourcephone.phone)
    end

    if (phones.empty?)
      return "NO_RESOURCE_PHONES"
    else
      return phones.join(", ")
    end
  end

  # ## ###  # ## ###  # ## ###  # ## ###  # ## ###  # ## ###
  # "Mix-in" the Fake module
  require 'crdb/fake.rb'
  include CRDB::Fake

  @@fake_method_to_value_hash = nil
  def (CRDB::Resourcephone).initialize_fake_method_to_value_hash()
    require 'crdb/resource'
    (CRDB::Resource).delete_all_fakes()
    resource = (CRDB::Resource).new()
    fake_resource0 = resource.make_fake({:index => 0})
    fake_resource1 = resource.make_fake({:index => 1})
    fake_resources = [fake_resource0, fake_resource1]

    fake_phones = ['650-555-5555', '650-555-12345']

    @@fake_method_to_value_hash = 
    {
      'resource' => fake_resources,
      'phone'    => fake_phones,
    }
  end

  def (CRDB::Resourcephone).get_fake_method_to_value_hash()
    return @@fake_method_to_value_hash
  end

  def (CRDB::Resourcephone).delete_all_fakes()
    # There should be two Resourcephone objects that we need to delete. 
    (CRDB::Resourcephone).initialize_fake_method_to_value_hash()
    fake_resources = @@fake_method_to_value_hash['resource']
    fake_phones    = @@fake_method_to_value_hash['phone']
    (CRDB::Resourcephone).delete_all(["resource_id = ? AND phone = ?", 
                                      fake_resources[0].id, fake_phones[0]])
    (CRDB::Resourcephone).delete_all(["resource_id = ? AND phone = ?", 
                                      fake_resources[1].id, fake_phones[1]])
     # And of course we delete the fake Resources:
    (CRDB::Resource).delete_all_fakes()
  end   
  # ## ###  # ## ###  # ## ###  # ## ###  # ## ###  # ## ###

 end # matches "class Resourcephone"
end # matches "module CRDB"

