require 'resource'

require 'crdb'

module CRDB
 class SundialInfo < Object

  @@ZM_RESOURCE_ACCOUNT_PREFIX = 'cal'
  @@ZM_RESOURCE_TYPE_PREFIX =
  {
    'location'          => 'lo',
    'equipment'         => 'eq',
    'shared_calendar'   => 'sc',
    'distribution_list' => 'dist',
    'unknown'           => 'xx',
  }

  @@SCRIPTS_HOME_DIR = '/afs/ir/dev/calendar/migration/scripts/'
  @@SUNDIAL_DATA_DIR = @@SCRIPTS_HOME_DIR + 'data/'

  attr_accessor :resource_id_to_rname,
                :resource_id_to_type,
                :resource_id_to_uid

 def initialize
   @resource_id_to_rname  = {}
   @resource_id_to_type   = {}
   @resource_id_to_uid = {}

   @@ZM_RESOURCE_TYPE_PREFIX.each do |type, prefix|
     filename = @@SUNDIAL_DATA_DIR + "resources/#{type}.txt"
     filehandle = File.new(filename, "r")

     uid_seen = {}

     while (line = filehandle.gets)
       # Skip blank lines.
       if (line =~ /^\s*$/)
         next
       end

       # Skip distribution lists.
       if (type =~ /dist/)
         next
       end

       rname, id, email = line.split(/,;,/)
       # puts "#{rname},#{id},#{email}"

       @resource_id_to_rname[id] = rname
       @resource_id_to_type[id]  = type

       uid = CRDB::SundialInfo.resource_name_to_uid(rname, type)
       @resource_id_to_uid[id] = uid
       # puts "uid is #{uid}"

       # Keep track of the zimbra emails to ensure there is no collision.
       if (uid_seen.key?(uid))
         other_rname = uid_seen[ui]
         raise "uid clash: '#{uid}' (#{rname}, #{other_rname})"
       else
         uid_seen[uid] = rname
       end
     end

     filehandle.close()
   end
 end


  # Convert the resource name to a Zimbra uid.
  def CRDB::SundialInfo.resource_name_to_uid(rname, type)
    # Run through the sanitizer in resource.rb
    sanitized_rname = (CRDB::Resource).sanitize_for_uid(rname)

    uid = @@ZM_RESOURCE_ACCOUNT_PREFIX + '_' +
          @@ZM_RESOURCE_TYPE_PREFIX[type] + '_' +
          sanitized_rname

    return uid
  end

 end # matches "module CRDB"
end # matches "class SundialInfo"
