module CRDB
 class Type < ActiveRecord::Base

  # This means every type has a category.
  belongs_to :category, :class_name => "CRDB::Category"
  has_many :resources, :class_name => "CRDB::Resource"

  ## VALIDATIONS
  validates_format_of :code, :with => /\A[A-Z][A-Z_]+\z/
  validates_uniqueness_of :code
  validates_uniqueness_of :description

  def to_s
    @id = self.id
    @code = self.code
    @description = self.description
    @category = self.category
    return "(id: #{@id}, code: #{@code}, description: #{@description}, category: #{@category.to_s})"
  end

  def category_code
    return self.category.code
  end

  # Return an array of all the type codes EXCEPT the unknown ones
  def (CRDB::Type).get_known_type_codes()
    types = (CRDB::Type).find(:all)

    type_codes = types.map do |type|
      if (type.code =~ /UNKNOWN/i)
        nil
      else
        type.code
      end
    end

    type_codes.compact!

    return type_codes
  end


  # ## ###  # ## ###  # ## ###  # ## ###  # ## ###  # ## ###
  # "Mix-in" the Fake module
  require 'crdb/fake.rb'
  include CRDB::Fake

  @@fake_method_to_value_hash = nil
  def (CRDB::Type).initialize_fake_method_to_value_hash()
    require 'crdb/category'
    (CRDB::Category).delete_all_fakes()
    obj_cat = (CRDB::Category).new()
    fake_categories = [
                        obj_cat.make_fake({:index => 0}),
                        obj_cat.make_fake({:index => 1}),
                      ]
    fake_codes = ['QZ', 'ZQ']
    fake_descriptions = ['a fake type', 'another fake type']
    @@fake_method_to_value_hash =
    {
      'category'    => fake_categories,
      'code'        => fake_codes,
      'description' => fake_descriptions,
    }
  end

  def (CRDB::Type).get_fake_method_to_value_hash()
    return @@fake_method_to_value_hash
  end

  def (CRDB::Type).delete_all_fakes()
    (CRDB::Type).initialize_fake_method_to_value_hash()
    fake_descriptions = @@fake_method_to_value_hash['description']
    (CRDB::Type).delete_all(["description = ?", fake_descriptions[0]])
    (CRDB::Type).delete_all(["description = ?", fake_descriptions[1]])
    (CRDB::Category).delete_all_fakes()
  end
  # ## ###  # ## ###  # ## ###  # ## ###  # ## ###  # ## ###

 end # matches "class Type"
end # matches "module CRDB"
