require 'zimbra'

require 'tempfile'

require 'crdb'

module CRDB
  class ZimbraCommand < Zimbra

  # Run a full zimbra command and return an array consisting of 
  # stdout and stderr.
  def run_command(command)
    stdin, stdout, stderr = Open3.popen3(command)

    # Dump output
    stdout_string = stdout.read
    stderr_string = stderr.read

    return [stdout_string, stderr_string]
  end


  # Given an array of commands, create a temporary file, 
  # store the commands in the temporary file, run either
  # zmprov or zmmailbox on the file, and return the results
  def run_several_commands(commands, zimbrabin)
   
    # Step 1. Create a temporary file.
    command_file = Tempfile.new('zimbracmd') 

    # Step 2. Store the commands in the command file
    command_file.print(commands.join("\n"))
    command_file.print("\n") # One extra
    command_file.flush()

    # Step 3. Run the commands
    cmd = "#{zimbrabin} -z -f #{command_file.path}" 
    stdout, stderr = self.run_command(cmd) 

    # Step 4. Remove the temporary file
    command_file.close
    command_file.unlink

    # Step 5. Return the results
    return [stdout, stderr]
  end

 end # matches "module CRDB"
end # matches "class ZimbraCommand"
